/*
 * Copyright 2011 Chris de Vreeze
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package eu.cdevreeze.springjdbc

import java.sql.{ ResultSet, Connection, PreparedStatement }
import org.springframework.jdbc.core.{ RowMapper, ConnectionCallback, PreparedStatementCallback }

/**
 * Implicit conversions that are useful in the context of a Spring
 * <code>JdbcTemplate</code>, to make the application code slightly more Scala-friendly.
 *
 * An implicit conversion makes it possible to invoke method toRowMapper on a function taking a
 * ResultSet and an Int (the row number), thus turning that function into a RowMapper.
 *
 * Another implicit conversion makes it possible to invoke method toCallback on a function taking a
 * Connection, thus turning that function into a ConnectionCallback. Analogously a function taking a
 * PreparedStatement can be turned into a PreparedStatementCallback.
 *
 * @author Chris de Vreeze
 */
object JdbcTemplateUtils {

  /** "Implicit" class containing the toRowMapper method. */
  final class ToRowMapper[T](f: (ResultSet, Int) => T) {
    def toRowMapper: RowMapper[T] =
      new RowMapper[T] {
        def mapRow(rs: ResultSet, rowNum: Int): T = f(rs, rowNum)
      }
  }

  /** "Implicit" class containing the toCallback method for functions taking a Connection. */
  final class ToConnectionCallback[T](f: (Connection) => T) {
    def toCallback: ConnectionCallback[T] =
      new ConnectionCallback[T] {
        def doInConnection(con: Connection): T = f(con)
      }
  }

  /** "Implicit" class containing the toCallback method for functions taking a PreparedStatement. */
  final class ToPreparedStatementCallback[T](f: (PreparedStatement) => T) {
    def toCallback: PreparedStatementCallback[T] =
      new PreparedStatementCallback[T] {
        def doInPreparedStatement(ps: PreparedStatement): T = f(ps)
      }
  }

  /**
   * Adds a toRowMapper method that implicitly converts a <code>(ResultSet, Int) => T</code>
   * function to a <code>RowMapper</code>.
   */
  implicit def toRowMapperConverter[T](f: (ResultSet, Int) => T): ToRowMapper[T] =
    new ToRowMapper[T](f)

  /**
   * Adds a toCallback method that implicitly converts a <code>(Connection) => T</code>
   * function to a <code>ConnectionCallback</code>.
   */
  implicit def toConnectionCallbackConverter[T](f: (Connection) => T): ToConnectionCallback[T] =
    new ToConnectionCallback[T](f)

  /**
   * Adds a toCallback method that implicitly converts a <code>(PreparedStatement) => T</code>
   * function to a <code>PreparedStatementCallback</code>.
   */
  implicit def toPreparedStatementCallbackConverter[T](f: (PreparedStatement) => T): ToPreparedStatementCallback[T] =
    new ToPreparedStatementCallback[T](f)
}
